#ifndef _CFREADER_BYTECODE_H
#define _CFREADER_BYTECODE_H

#include "cfreader_global.h"
#include "cfreader_methods.h"
#include "cfreader_attributes.h"

/* Bytecode mnemonics */
#define JBC_NOP             ((u1)   0)
#define JBC_ACONST_NULL     ((u1)   1)
#define JBC_ICONST_M1       ((u1)   2)
#define JBC_ICONST_0        ((u1)   3)
#define JBC_ICONST_1        ((u1)   4)
#define JBC_ICONST_2        ((u1)   5)
#define JBC_ICONST_3        ((u1)   6)
#define JBC_ICONST_4        ((u1)   7)
#define JBC_ICONST_5        ((u1)   8)
#define JBC_LCONST_0        ((u1)   9)
#define JBC_LCONST_1        ((u1)  10)
#define JBC_FCONST_0        ((u1)  11)
#define JBC_FCONST_1        ((u1)  12)
#define JBC_FCONST_2        ((u1)  13)
#define JBC_DCONST_0        ((u1)  14)
#define JBC_DCONST_1        ((u1)  15)
#define JBC_BIPUSH          ((u1)  16)
#define JBC_SIPUSH          ((u1)  17)
#define JBC_LDC             ((u1)  18)
#define JBC_LDC_W           ((u1)  19)
#define JBC_LDC2_W          ((u1)  20)
#define JBC_ILOAD           ((u1)  21)  
#define JBC_LLOAD           ((u1)  22)
#define JBC_FLOAD           ((u1)  23)
#define JBC_DLOAD           ((u1)  24)
#define JBC_ALOAD           ((u1)  25)
#define JBC_ILOAD_0         ((u1)  26)
#define JBC_ILOAD_1         ((u1)  27)
#define JBC_ILOAD_2         ((u1)  28)
#define JBC_ILOAD_3         ((u1)  29)
#define JBC_LLOAD_0         ((u1)  30)
#define JBC_LLOAD_1         ((u1)  31)
#define JBC_LLOAD_2         ((u1)  32)
#define JBC_LLOAD_3         ((u1)  33)
#define JBC_FLOAD_0         ((u1)  34)
#define JBC_FLOAD_1         ((u1)  35)
#define JBC_FLOAD_2         ((u1)  36)
#define JBC_FLOAD_3         ((u1)  37)
#define JBC_DLOAD_0         ((u1)  38)
#define JBC_DLOAD_1         ((u1)  39)
#define JBC_DLOAD_2         ((u1)  40)
#define JBC_DLOAD_3         ((u1)  41)
#define JBC_ALOAD_0         ((u1)  42)
#define JBC_ALOAD_1         ((u1)  43)
#define JBC_ALOAD_2         ((u1)  44)
#define JBC_ALOAD_3         ((u1)  45)
#define JBC_IALOAD          ((u1)  46)
#define JBC_LALOAD          ((u1)  47)
#define JBC_FALOAD          ((u1)  48)
#define JBC_DALOAD          ((u1)  49)
#define JBC_AALOAD          ((u1)  50)
#define JBC_BALOAD          ((u1)  51)
#define JBC_CALOAD          ((u1)  52)
#define JBC_SALOAD          ((u1)  53)
#define JBC_ISTORE          ((u1)  54)
#define JBC_LSTORE          ((u1)  55)
#define JBC_FSTORE          ((u1)  56)
#define JBC_DSTORE          ((u1)  57)
#define JBC_ASTORE          ((u1)  58)
#define JBC_ISTORE_0        ((u1)  59)
#define JBC_ISTORE_1        ((u1)  60)
#define JBC_ISTORE_2        ((u1)  61)
#define JBC_ISTORE_3        ((u1)  62)
#define JBC_LSTORE_0        ((u1)  63)
#define JBC_LSTORE_1        ((u1)  64)
#define JBC_LSTORE_2        ((u1)  65)
#define JBC_LSTORE_3        ((u1)  66)
#define JBC_FSTORE_0        ((u1)  67)
#define JBC_FSTORE_1        ((u1)  68)
#define JBC_FSTORE_2        ((u1)  69)
#define JBC_FSTORE_3        ((u1)  70)
#define JBC_DSTORE_0        ((u1)  71)
#define JBC_DSTORE_1        ((u1)  72)
#define JBC_DSTORE_2        ((u1)  73)
#define JBC_DSTORE_3        ((u1)  74)
#define JBC_ASTORE_0        ((u1)  75)
#define JBC_ASTORE_1        ((u1)  76)
#define JBC_ASTORE_2        ((u1)  77)
#define JBC_ASTORE_3        ((u1)  78)
#define JBC_IASTORE         ((u1)  79)
#define JBC_LASTORE         ((u1)  80)
#define JBC_FASTORE         ((u1)  81)
#define JBC_DASTORE         ((u1)  82)
#define JBC_AASTORE         ((u1)  83)
#define JBC_BASTORE         ((u1)  84)
#define JBC_CASTORE         ((u1)  85)
#define JBC_SASTORE         ((u1)  86)
#define JBC_POP             ((u1)  87)
#define JBC_POP2            ((u1)  88)
#define JBC_DUP             ((u1)  89)
#define JBC_DUP_X1          ((u1)  90)
#define JBC_DUP_X2          ((u1)  91)
#define JBC_DUP2            ((u1)  92)
#define JBC_DUP2_X1         ((u1)  93)
#define JBC_DUP2_X2         ((u1)  94)
#define JBC_SWAP            ((u1)  95)
#define JBC_IADD            ((u1)  96)
#define JBC_LADD            ((u1)  97)
#define JBC_FADD            ((u1)  98)
#define JBC_DADD            ((u1)  99)
#define JBC_ISUB            ((u1) 100)
#define JBC_LSUB            ((u1) 101)
#define JBC_FSUB            ((u1) 102)
#define JBC_DSUB            ((u1) 103)
#define JBC_IMUL            ((u1) 104)
#define JBC_LMUL            ((u1) 105)
#define JBC_FMUL            ((u1) 106)
#define JBC_DMUL            ((u1) 107)
#define JBC_IDIV            ((u1) 108)
#define JBC_LDIV            ((u1) 109)
#define JBC_FDIV            ((u1) 110)
#define JBC_DDIV            ((u1) 111)
#define JBC_IREM            ((u1) 112)
#define JBC_LREM            ((u1) 113)
#define JBC_FREM            ((u1) 114)
#define JBC_DREM            ((u1) 115)
#define JBC_INEG            ((u1) 116)
#define JBC_LNEG            ((u1) 117)
#define JBC_FNEG            ((u1) 118)
#define JBC_DNEG            ((u1) 119)
#define JBC_ISHL            ((u1) 120)
#define JBC_LSHL            ((u1) 121)
#define JBC_ISHR            ((u1) 122)
#define JBC_LSHR            ((u1) 123)
#define JBC_IUSHR           ((u1) 124)
#define JBC_LUSHR           ((u1) 125)
#define JBC_IAND            ((u1) 126)
#define JBC_LAND            ((u1) 127)
#define JBC_IOR             ((u1) 128)
#define JBC_LOR             ((u1) 129)
#define JBC_IXOR            ((u1) 130)
#define JBC_LXOR            ((u1) 131)
#define JBC_IINC            ((u1) 132)
#define JBC_I2L             ((u1) 133)
#define JBC_I2F             ((u1) 134)
#define JBC_I2D             ((u1) 135)
#define JBC_L2I             ((u1) 136)
#define JBC_L2F             ((u1) 137)
#define JBC_L2D             ((u1) 138)
#define JBC_F2I             ((u1) 139)
#define JBC_F2L             ((u1) 140)
#define JBC_F2D             ((u1) 141)
#define JBC_D2I             ((u1) 142)
#define JBC_D2L             ((u1) 143)
#define JBC_D2F             ((u1) 144)
#define JBC_I2B             ((u1) 145)
#define JBC_I2C             ((u1) 146)
#define JBC_I2S             ((u1) 147)
#define JBC_LCMP            ((u1) 148)
#define JBC_FCMPL           ((u1) 149)
#define JBC_FCMPG           ((u1) 150)
#define JBC_DCMPL           ((u1) 151)
#define JBC_DCMPG           ((u1) 152)
#define JBC_IFEQ            ((u1) 153)
#define JBC_IFNE            ((u1) 154)
#define JBC_IFLT            ((u1) 155)
#define JBC_IFGE            ((u1) 156)
#define JBC_IFGT            ((u1) 157)
#define JBC_IFLE            ((u1) 158)
#define JBC_IF_ICMPEQ       ((u1) 159)
#define JBC_IF_ICMPNE       ((u1) 160)
#define JBC_IF_ICMPLT       ((u1) 161)
#define JBC_IF_ICMPGE       ((u1) 162)
#define JBC_IF_ICMPGT       ((u1) 163)
#define JBC_IF_ICMPLE       ((u1) 164)
#define JBC_IF_ACMPEQ       ((u1) 165)
#define JBC_IF_ACMPNE       ((u1) 166)
#define JBC_GOTO            ((u1) 167)
#define JBC_JSR             ((u1) 168)
#define JBC_RET             ((u1) 169)
#define JBC_TABLESWITCH     ((u1) 170)
#define JBC_LOOKUPSWITCH    ((u1) 171)
#define JBC_IRETURN         ((u1) 172)
#define JBC_LRETURN         ((u1) 173)
#define JBC_FRETURN         ((u1) 174)
#define JBC_DRETURN         ((u1) 175)
#define JBC_ARETURN         ((u1) 176)
#define JBC_RETURN          ((u1) 177)
#define JBC_GETSTATIC       ((u1) 178)
#define JBC_PUTSTATIC       ((u1) 179)
#define JBC_GETFIELD        ((u1) 180)
#define JBC_PUTFIELD        ((u1) 181)
#define JBC_INVOKEVIRTUAL   ((u1) 182)
#define JBC_INVOKESPECIAL   ((u1) 183)
#define JBC_INVOKESTATIC    ((u1) 184)
#define JBC_INVOKEINTERFACE ((u1) 185)
#define JBC_XXXUNUSEDXXX1   ((u1) 186)
#define JBC_NEW             ((u1) 187)
#define JBC_NEWARRAY        ((u1) 188)
#define JBC_ANEWARRAY       ((u1) 189)
#define JBC_ARRAYLENGTH     ((u1) 190)
#define JBC_ATHROW          ((u1) 191)
#define JBC_CHECKCAST       ((u1) 192)
#define JBC_INSTANCEOF      ((u1) 193)
#define JBC_MONITORENTER    ((u1) 194)
#define JBC_MONITOREXIT     ((u1) 195)
#define JBC_WIDE            ((u1) 196)
#define JBC_MULTIANEWARRAY  ((u1) 197)
#define JBC_IFNULL          ((u1) 198)
#define JBC_IFNONNULL       ((u1) 199)
#define JBC_GOTO_W          ((u1) 200)
#define JBC_JSR_W           ((u1) 201)
#define JBC_BREAKPOINT      ((u1) 202)
#define JBC_IMPDEP1         ((u1) 254)
#define JBC_IMPDEP2         ((u1) 255)

typedef struct bytecode {
    u1 *code;
    u4 length;
} cfrt_bytecode;

int cfrf_get_bytecode(cfrt_method_info *method, cfrt_bytecode *bytecode);
u4 cfrf_predict_next_offset(cfrt_bytecode *bytecode, u4 offset);

#endif
